/* TMSNC - Textbased MSN Client
 * Copyright (C) 2004 The IR Developer Group
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the IR Public Domain License as published by
 *  the IR Group; either version 1.6 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the IR Public Domain License
 * along with this program; if not, write to sanoix@gmail.com.
 */

#include "core_misc.h"
#include "config.h"
#include "core.h"

void MSN_init(session)
    MSN_session *session;
{	
  int i;

  session->gtc=0x0;
  session->blp=0x0;
  session->num_contacts=0;
  session->csc=0;
  session->me.addr[0]=0x0;
  session->me.name[0]=0x0;
  session->me.listnum=0;
  session->me.status=0;
  strncpy(session->format.color, "000000", 6);
  strncpy(session->format.font, "Arial", 5);
  session->format.effect=' ';

  for(i=0;i<MAX_CONTACTS;i++)
  {
    memset(session->contact[i].addr, 0x0, sizeof(session->contact[i].addr));
    memset(session->contact[i].name, 0x0, sizeof(session->contact[i].name));
    memset(session->contact[i].custom_name, 0x0, sizeof(session->contact[i].custom_name));
  }
}

char *stringdup(str)
    char *str;
{
  char *new;
  new = (char *)malloc((strlen(str)+1)*sizeof(char));
  if(new==NULL)
    return NULL;
  strcpy(new, str);
  return new;
}

/* does the same thing as fgets() but does not */
/* require a (FILE *) pointer                  */
int getline(str, size, sd)
    char *str;
    int size;
    int sd;
{
  char c, *ptr=str;
  int r;

  while((r = read(sd, &c, 1))!=0)
  {
    if(r<0)
      return -1;

    if((ptr-str)>=size)
      break;

    *ptr = c;
    ptr++;

    if(c=='\n')
      break;
  }

  *ptr = '\0';
  return (ptr-str);
}

char *split(str, split, ret)
    char *str;
    char split;
    int ret;
{
  char *ptr;
  int a, b, c=0;

  /* Find the first split-char */
  if(ret!=0)
  {
    for(a=0;a<=strlen(str);a++)
    {
      if(str[a]==split && ++c==ret) 
      { 
        a++; 
        break; 
      }
      if(a==strlen(str)) 
        return NULL;
    }
  } else 
  {
    a = 0;
  }
  
  /* Find the second split-char */
  for(b=a;b<=strlen(str);b++)
    if(str[b]==split) break;
  
  ptr = (char *)malloc(sizeof(char)*(b-a+1));
  strncpy(ptr, &str[a], b-a);
  ptr[b-a]=0x0;
  
  return (char *)ptr;
}

/* Check if addr is a valid e-mail address */
int MSN_is_valid_address(addr)
     char *addr;
{
  int i;
  char *ptr;
  for(i=0;i<strlen(addr);i++) 
    if(isspace(addr[i])) return -1;
  if((ptr = strchr(addr, '@'))==NULL) return -1;
  if((ptr = strchr(addr, '.'))==NULL) return -1;
  return 0;
}

char *MSN_url_encode(str, ret, ret_len)
     char *str;
     char *ret;
     int ret_len;
{
  int i, a;
  for(i=0, a=0;i<strlen(str) && a<ret_len;i++, a++)
  {
    if(!isalnum(str[i]))
    {
      snprintf(&ret[a], ret_len-i, "%%%x", str[i]);
      a+=2;
    }
    else
    {
      ret[a] = str[i];
    }
  }
  ret[a]=0x0;
  return ret;
}

char *MSN_url_decode(str, ret, retsize)
     char *str;
     char *ret;
     int retsize;
{
  char buf[retsize+1];
  char hex[3];
  int x, y;

  for (x=0, y=0;x<strlen(str) && y<retsize; x++, y++)
  {
    if(str[x]=='%' && x<strlen(str)-2)
    {
      strncpy(hex, &str[x+1], 2);
      buf[y]=(char)strtol(hex, NULL, 16);
      if(buf[y]=='\t' || buf[y]=='\n' || buf[y]=='\r')
        buf[y]=' ';
      x+=2;
    } 
    else
    {
      buf[y]=str[x];
    }
  }
  buf[y]='\0';
  strncpy(ret, buf, retsize);
  
  return ret;
}

void MSN_error2str(err, message, message_len)
     int err;
     char *message;
     int message_len;
{  
  char buf[128];
  switch(err)
  {
    case 200: 
      strcpy(buf, _("Invalid syntax")); 
      break;
    case 201: 
      strcpy(buf, _("Invalid parameter")); 
      break;
    case 205: 
      strcpy(buf, _("Invalid principal")); 
      break;
    case 206: 
      strcpy(buf, _("Domain name missing")); 
      break;
    case 207: 
      strcpy(buf, _("Already logged in")); 
      break;
    case 208: 
      strcpy(buf, _("Invalid principal-name"));
      break;
    case 209:
      strcpy(buf, _("Nickname change illegal"));
      break;
    case 210:
      strcpy(buf, _("Principal list full"));
      break;
    case 215:
      strcpy(buf, _("Principal already on list"));
      break;
    case 216: 
      strcpy(buf, _("Principal not on list"));
      break;
    case 217:
      strcpy(buf, _("Principal not on-line")); 
      break;
    case 218: 
      strcpy(buf, _("Already in mode")); 
      break;
    case 219:
      strcpy(buf, _("Principal is in the opposite list")); 
      break;
    case 280:
      strcpy(buf, _("Switchboard failed"));
      break;
    case 281:
      strcpy(buf, _("Transfer to switchboard failed")); 
      break;
      
    case 300: 
      strcpy(buf, _("Required field missing")); 
      break;
    case 302: 
      strcpy(buf, _("Not logged in")); 
      break;
      
    case 500: 
      strcpy(buf, _("Internal server error")); 
      break;
    case 501: 
      strcpy(buf, _("Database server error")); 
      break;
    case 502: 
      strcpy(buf, _("Command disabled")); 
      break;
    case 510: 
      strcpy(buf, _("File operation failed"));
      break;
    case 520:
      strcpy(buf, _("Memory allocation failed"));
      break;
    case 540: 
      strcpy(buf, _("Challenge response failed")); 
      break;
      
    case 602: 
      strcpy(buf, _("Peer nameserver is down")); 
      break;
    case 603: 
      strcpy(buf, _("Database connection failed")); 
      break;
    case 604: 
      strcpy(buf, _("Server is going down")); 
      break;
      
    case 707: 
      strcpy(buf, _("Could not create connection")); 
      break;
    case 710: 
      strcpy(buf, _("Bad CVR parameters sent")); 
      break;
    case 711: 
      strcpy(buf, _("Write is blocking"));
      break;
    case 712:
      strcpy(buf, _("Session is overloaded")); 
      break;
    case 714:
      strcpy(buf, _("Too many sessions"));
      break;
    case 715: 
    case 731: 
      strcpy(buf, _("Not expected"));
      break;
    case 717:
      strcpy(buf, _("Bad friend file"));
      break;
      
    case 800: 
    case 713: 
      strcpy(buf, _("Changing too rapidly")); 
      break;
      
    case 600: 
    case 910: 
    case 912: 
    case 918: 
    case 919: 
    case 921: 
    case 922:
      strcpy(buf, _("Server too busy")); 
      break;
    case 911:
    case 917: 
      strcpy(buf, _("Autentication failed"));
      break;
    case 913:
      strcpy(buf, _("Not allowed when hiding"));
      break;
    case 914:
    case 915:
    case 916:
    case 605: 
    case 601:
      strcpy(buf, _("Server unavailable")); 
      break;
    case 920: 
      strcpy(buf, _("Not accepting new principals"));
      break;
    case 923: 
      strcpy(buf, _("Kids Passport without parental consent"));
      break;
    case 924: 
      strcpy(buf, _("Passport account not yet verified")); 
      break;
    case 928: 
      strcpy(buf, _("Bad ticket")); 
      break;
    default: 
      strcpy(buf, _("Unknown error")); 
      break;
  }
  strncpy(message, buf, message_len);
}
