/* TMSNC - Textbased MSN Client
 * Copyright (C) 2004 The IR Developer Group
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the IR Public Domain License as published by
 *  the IR Group; either version 1.6 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the IR Public Domain License
 * along with this program; if not, write to sanoix@gmail.com.
 */

#include "event.h"
#include "common.h"

int UI_handle_server_code(ui, session, c, str, cf)
    struct ui *ui;
    MSN_session *session;
    int c;
    char *str;
    config *cf;
{
  char *ptr[2]={NULL, NULL}, t_buf[16];
  int r;

  switch(c)
  {
    case -2: /* fatal error */
      UI_play_sound(cf, SOUND_ERROR);
      UI_err_exit(_("Quitting TMSNC: %s\n"), str);
      break;
    case 2:  /* initial login */
    case 6:  /* someone logged out */
    case 11: /* added contact */
    case 12: /* removed contact */
    case 13:  /* someone changed status */
      UI_draw_list(ui, session, cf);
      break;
    case 8: /* we are invited to a conversation */
      if((ptr[0] = split(str, ' ', 0))==NULL ||
         (ptr[1] = split(str, ' ', 1))==NULL)
        UI_err_exit(_("Cannot split conversation string"));
      r = UI_start_conversation(ui,
                                session,
                                cf,
                                ptr[1],
                                atoi(ptr[0]));
      free(ptr[0]);
      free(ptr[1]);

      if(r<0)
      {
        wattron(ui->main.swin, COLOR_PAIR(1) | A_BOLD); 
        UI_win_addstr(&ui->main,"[%s] ",UI_get_timestr(t_buf, sizeof(t_buf)-1));
        wattroff(ui->main.swin, COLOR_PAIR(1) | A_BOLD); 
        UI_win_addstr(&ui->main, _("[%s] Error: Cannot accept conversation\n"));
        UI_play_sound(cf, SOUND_ERROR);
      }
      UI_draw_conv(ui, cf);
      break;
    case 9: /* we changed screen-name */
      UI_bar_draw(session);
      break;
    case 14: /* someone logged in */
      UI_play_sound(cf, SOUND_LOGIN);
      UI_draw_list(ui, session, cf);
      break;
    default:
      break;
  }
  return 0;
}

int UI_main_handle_key(ui, session, cf, c)
    struct ui *ui;
    MSN_session *session;
    config *cf;
    int c;
{
  char t_buf[16], buf[1024];
  int width, height, i;

  #ifdef ENABLE_NLS
    size_t field_buf_size;
  #endif

  width  = ui->c_win->x-4;
  height = 3;

  switch(c)
  {
    case KEY_LEFT:
      if(ui->c_win->field_ptr>0)
        ui->c_win->field_ptr--;
      break;
    case KEY_RIGHT:
      if(ui->c_win->field_ptr < ui->c_win->field_length)
        ui->c_win->field_ptr++;
      break;
    case KEY_BACKSPACE:
    case 127: /* Backspace in terminal.app */
      if(ui->c_win->field_ptr>0)
      {
        ui->c_win->field_ptr--;
        ui->c_win->field_length--;
        for(i=ui->c_win->field_ptr;i<ui->c_win->field_length;i++)
          ui->c_win->field_buf[i] = ui->c_win->field_buf[i+1];
        ui->c_win->field_buf[i] = 0x0;
      }
      break;
    case '\n':
    case '\r':
      /* Break on empty buffer */
      if(ui->c_win->field_buf[0]==0x0)
        break;

      if(ui->c_win == &ui->main) // we're in the console
      {
        UI_win_addstr(&ui->main, _("No commands implemeted at the moment. Sorry :)\n"));

        /* Clean the input field */
        ui->c_win->field_buf[0]=0x0;
        ui->c_win->field_ptr=0;
        ui->c_win->field_length=0;
        werase(ui->c_win->field_win);
        break;
      }

      if(ui->c_conv->num_ppl<=0)
      {
        for(i=0;i<session->num_contacts &&
                strcmp(session->contact[i].addr,ui->c_conv->contact)==0;i++);
        if(i==session->num_contacts) return -1;
        if(session->contact[i].status!=7)
        {
          MSN_conversation_call(ui->c_conv->sd, ui->c_conv->csc++, ui->c_conv->contact);

          /* to give the server some time to process our invitation */
          sleep(1);

          if(MSN_conversation_handle(ui->c_conv->sd, &(ui->c_conv->num_ppl), buf, sizeof(buf)-1)>=1)
            UI_win_addstr(ui->c_win, "[%s] %s\n",UI_get_timestr(t_buf, sizeof(t_buf)-1),buf);
        }
      }

      /* Run the buffer through a talkfilter if available */
      UI_filter_translate(cf, ui->c_win->field_buf, FIELDBUFSIZE-1);

      #ifdef ENABLE_NLS
        field_buf_size = (size_t)FIELDBUFSIZE;
        UI_convert_to_utf8(ui->c_win->field_buf,
                           field_buf_size,
                           buf,sizeof(buf));
      #endif

      /* Send the message */
      if(MSN_conversation_sendmsg(ui->c_conv->sd,
                                  ui->c_conv->csc++,
      #ifdef ENABLE_NLS
                                  buf,
      #else
                                  ui->c_win->field_buf,
      #endif
                                  session)==-1)
      {
        ui->c_conv->num_ppl=0;

        /* Restart conversation */
        if(UI_start_conversation(ui, session, cf, ui->c_conv->contact, 0)>0)
        {
          if(MSN_conversation_handle(ui->c_conv->sd, &(ui->c_conv->num_ppl), buf, sizeof(buf)-1)>=1)
          {
            wattron(ui->c_win->swin, COLOR_PAIR(3));
            UI_win_addstr(ui->c_win, "[%s]",UI_get_timestr(t_buf, sizeof(t_buf)-1));
            wattroff(ui->c_win->swin, COLOR_PAIR(3)); 
            UI_win_addstr(ui->c_win, " %s\n", buf);
          }
        }
        else
        {
            wattron(ui->main.swin, COLOR_PAIR(1) | A_BOLD); 
            UI_win_addstr(&ui->main,"[%s] ",UI_get_timestr(t_buf, sizeof(t_buf)-1));
            wattroff(ui->main.swin, COLOR_PAIR(1) | A_BOLD); 
            UI_win_addstr(ui->c_win, _("Error: Could not restart conversation\n"),UI_get_timestr(t_buf, sizeof(t_buf)-1));
        }
      }
      wattron(ui->c_win->swin, COLOR_PAIR(3));
      UI_win_addstr(ui->c_win, "[%s]",UI_get_timestr(t_buf, sizeof(t_buf)-1));
      wattroff(ui->c_win->swin, COLOR_PAIR(3));

      #ifdef ENABLE_NLS
        UI_convert_from_utf8(session->me.name,
                             sizeof(session->me.name),
                             buf,sizeof(buf));
      #else
        strncpy(buf, session->me.name, sizeof(buf)-1);
      #endif

      UI_win_addstr(ui->c_win, _(" %s says:\n     %s\n"),
                               buf,
                               ui->c_win->field_buf);
      if(ui->c_conv->logfd!=NULL)
      {
        fprintf(ui->c_conv->logfd, _("[%s] %s says:\n     %s\n"),
                                   UI_get_timestr(t_buf, sizeof(t_buf)-1),
                                   buf,
                                   ui->c_win->field_buf);
      }
      ui->c_win->field_buf[0]=0x0;
      ui->c_win->field_ptr=0;
      ui->c_win->field_length=0;
      werase(ui->c_win->field_win);
      break;
    case -1:
    case KEY_RESIZE:
    case KEY_UP:
    case KEY_DOWN:
      return -1;
      break;
    default:
      if(ui->c_win->field_length>=FIELDBUFSIZE-1) return -1;

      for(i=ui->c_win->field_length;i>ui->c_win->field_ptr && i>0;i--)
        ui->c_win->field_buf[i] = ui->c_win->field_buf[i-1];

      ui->c_win->field_buf[ui->c_win->field_ptr]=c;
      ui->c_win->field_buf[ui->c_win->field_length+1]=0x0;
      ui->c_win->field_ptr++;
      ui->c_win->field_length++;
      break;
  }
  UI_win_put_buffer(ui->c_win->field_win,
                    ui->c_win->field_buf,
                    height,width,
                    ui->c_win->field_ptr);
  wrefresh(ui->c_win->field_win);
  return 0;
}

int UI_do_command(session, cf, ui, c, ret, retsize)
    MSN_session *session;
    config *cf;
    struct ui *ui;
    int c;
    char *ret;
    size_t retsize;
{
  int r;
  char buf[512], r_buf[512], *ptr;

  switch(c)
  {
    case cmd_none:
      break;
    case cmd_statusonline:
      if(MSN_set_status(session, "online")<0)
        UI_dialog_getch(_("Cannot set status to 'online'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statusaway:
      if(MSN_set_status(session, "away")<0)
        UI_dialog_getch(_("Cannot set status to 'away'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statusidle:
      if(MSN_set_status(session, "idle")<0)
        UI_dialog_getch(_("Cannot set status to 'idle'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statusbusy:
      if(MSN_set_status(session, "busy")<0)
        UI_dialog_getch(_("Cannot set status to 'busy'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statusbrb:
      if(MSN_set_status(session, "brb")<0)
        UI_dialog_getch(_("Cannot set status to 'be right back'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statusphone:
      if(MSN_set_status(session, "phone")<0)
        UI_dialog_getch(_("Cannot set status to 'on phone'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statuslunch:
      if(MSN_set_status(session, "lunch")<0)
        UI_dialog_getch(_("Cannot set status to 'on lunch'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_statushidden:
      if(MSN_set_status(session, "hidden")<0)
        UI_dialog_getch(_("Cannot set status to 'hidden'"));
      else
        UI_bar_draw(session);
      break;
    case cmd_changenick:
      if(UI_dialog_getstr(_("Enter new nickname"), buf, sizeof(buf)-1)==0)
      {
        #ifdef ENABLE_NLS
          strncpy(r_buf, buf, sizeof(r_buf)-1);
          UI_convert_to_utf8(r_buf,sizeof(r_buf),buf,sizeof(buf));
        #endif

        MSN_change_nick(buf, session);
      }
      break;
    case cmd_setfilter:
#ifdef HAVE_TALKFILTERS
      if(UI_dialog_getstr(_("Name of filter ('none' to disable)"), buf, sizeof(buf)-1)==0)
        if(UI_set_filter(cf, buf)<0)
          UI_dialog_getch(_("No such filter, read the manual"));
#else
      UI_dialog_getch(_("TMSNC was not compiled with talkfilter support"));
#endif
      break;
    case cmd_addcontact:
      if(UI_dialog_getstr(_("Add contact"), buf, sizeof(buf)-1)==0)
        if(MSN_add_contact(buf, session, r_buf, sizeof(r_buf)-1)<0)
          UI_dialog_getch(r_buf);
      break;
    case cmd_removecontact:
      if(MSN_remove_contact(session->contact[UI_get_list_pointer()].addr, session, r_buf, sizeof(r_buf)-1)<0)
        UI_dialog_getch(r_buf);
      break;
    case cmd_blockcontact:
      if(MSN_block_contact(session->contact[UI_get_list_pointer()].addr, session, r_buf, sizeof(r_buf)-1)<0)
        UI_dialog_getch(r_buf);
      break;
    case cmd_unblockcontact:
      if(MSN_unblock_contact(session->contact[UI_get_list_pointer()].addr, session, r_buf, sizeof(r_buf)-1)<0)
        UI_dialog_getch(r_buf);
      break;
    case cmd_console:
      UI_top_panel(ui, &ui->main);
      ui->c_conv = NULL;
      break;
    case cmd_redraw:
      UI_manual_resize();
      break;
    case cmd_setcustomnick:
      ptr = UI_get_contact_name(&(session->contact[UI_get_list_pointer()]),
                                cf->use_nickname);
      snprintf(r_buf,sizeof(r_buf)-1,
               _("New custom nickname for %s"), ptr);
      free(ptr);

      if(UI_dialog_getstr(r_buf, buf, sizeof(buf)-1)==0)
        UI_set_alias(&(session->contact[UI_get_list_pointer()]),buf);
      UI_draw_list(ui,session,cf);
      break;
    case cmd_unsetcustomnick:
      UI_delete_alias(&(session->contact[UI_get_list_pointer()]));
      UI_draw_list(ui,session,cf);
      break; 
    case cmd_manual:
      def_prog_mode();
      endwin();
      if((r=fork())==0)
      {
        r = execlp("man", "man", "tmsnc", NULL);
        reset_prog_mode();
        refresh();
        if(r<0)
        {
          strncpy(ret, strerror(errno), retsize);
          return -1;
        }
      }
      else if(r>0)
        wait(0);
      break;
    default:
      break;
  }

  return 0;
}

//Start of iveqy
int UI_handle_power_functions(ui, session, cf)
    struct ui *ui;
    MSN_session *session;
    config *cf;
{
  int i;
  char r_buf[512];

  UI_dialog_getstr(_("Enter command"), r_buf, sizeof(r_buf)-1);

  if(!strcasecmp("block all", r_buf))
  {
    if(UI_dialog_getch(_("Block entire contact list? (y/n)")) == 'y')
    {
      for(i=0; i < session->num_contacts; i++)
        MSN_block_contact(session->contact[i].addr, session,r_buf, sizeof(r_buf)-1);
    }
  }
  else if(!strcasecmp("unblock all", r_buf))
  {
    if(UI_dialog_getch(_("Unblock entire contact list? (y/n)")) == 'y')
    {
      for(i=0; i < session->num_contacts; i++)
        MSN_unblock_contact(session->contact[i].addr, session,r_buf, sizeof(r_buf)-1);
    }
  }
  else if(!strcasecmp("remove all", r_buf))
  {
    if(UI_dialog_getch(_("Remove entire contact list? (y/n)")) == 'y')
    {
      for(i=0; i < session->num_contacts; i++)
        MSN_block_contact(session->contact[i].addr, session,r_buf, sizeof(r_buf)-1);
    }
  }
  else if(!strcasecmp("whoami", r_buf))
  {
    snprintf(r_buf, sizeof(r_buf)-1, _("Your email address is: %s"), session->me.addr);
    UI_dialog_getch(r_buf);
  }
  else if(!strcasecmp("export conlist", r_buf))
  {
    if(UI_dialog_getstr(_("Export contact list to file"),r_buf, sizeof(r_buf)-1)==0)
    {
      FILE *outp;

      outp = fopen(r_buf, "w");

      if(outp == NULL)
        UI_dialog_getch(_("Error, trying to open file"));
      else
      {
        for(i = 0; i < session->num_contacts; i++)
          fprintf(outp, "%s\n", session->contact[i].addr);
        fclose(outp);
        UI_dialog_getch(_("Export completeted, press any key"));
      }
    }
  }
  else if(!strcasecmp("import conlist", r_buf))
  {
    if(UI_dialog_getstr(_("Import contact list to file"), r_buf, sizeof(r_buf)-1)==0)
    {
      FILE *inp;
      char *ptr, buf[512];
      inp = fopen(r_buf, "r");

      while(fgets(buf, sizeof(buf)-1, inp) != NULL)
      {
        if((ptr=strchr(buf, '\n'))!=NULL) *ptr='\0';
        MSN_add_contact(buf, session, r_buf, sizeof(r_buf)-1);
      }
      fclose(inp);
    }
  }
  return 0;
}

int UI_handle_altkey(ui, menu, cf, focus)
    struct ui *ui;
    struct menu *menu;
    config *cf;
    int *focus;
{
  int c;
  switch((c=tolower(getch())))
    {
     case 'a':
       UI_remove_focus(ui, menu, cf, *focus);
       *focus = 1;
       UI_set_focus(ui, menu, cf, *focus);
       break;
     case 's':
       UI_remove_focus(ui, menu, cf, *focus);
       *focus = 2;
       UI_set_focus(ui, menu, cf, *focus);
       break;
     case 'x':
       UI_remove_focus(ui, menu, cf, *focus);
       *focus = 3;
       UI_set_focus(ui, menu, cf, *focus);
       break;
     case 'z':
       UI_remove_focus(ui, menu, cf, *focus);
       *focus = 4;
       UI_set_focus(ui, menu, cf, *focus);
       break;
    case 'c':
      /* close current conversation */
      if(ui->c_conv!=NULL)
      {
        UI_close_conversation(ui, cf, ui->c_conv);
      }
      break;
    default:
      if(c>='1' && c<='9')
      {
        c-='0';
        if(c>ui->num_conversations) break;
        c--;

        UI_top_panel(ui, &(ui->conv[c].win));
        ui->c_conv = &(ui->conv[c]);
        ui->conv[c].new_message=0;

        UI_draw_conv(ui, cf); 
      }
      break;
    }
  return 0;
}
//End of iveqy
