/* TMSNC - Textbased MSN Client
 * Copyright (C) 2004 The IR Developer Group
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the IR Public Domain License as published by
 *  the IR Group; either version 1.6 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the IR Public Domain License
 * along with this program; if not, write to sanoix@gmail.com.
 */

#include "login.h"

void callback(i, str)
    int i;
    char *str;
{
  werase(statuswin);
  if(i!=-1)
  {
    mvwaddnstr(statuswin, 1, 3,
             "|||||||||||||||||||||||||||||||||", i*3);
  }
  else
    mvwaddstr(statuswin, 1, 18, _("ERROR"));

  mvwaddch(statuswin, 1, 2, '[');
  mvwaddch(statuswin, 1, 38, ']');
  mvwaddstr(statuswin, 3, (42-(strlen(str)))/2, str);
  wrefresh(statuswin);
}

void print_logo(win)
    WINDOW *win;
{
  int row=2, col=1, i;

  char logo[]=".%%%%%%..%%...%%...%%%%...%%..%%...%%%%.\n"
              "...%%....%%%.%%%..%%......%%%.%%..%%..%%\n"
              "...%%....%%.%.%%...%%%%...%%.%%%..%%....\n"
              "...%%....%%...%%......%%..%%..%%..%%..%%\n"
              "...%%....%%...%%...%%%%...%%..%%...%%%%.";

  wattron(win, COLOR_PAIR(4) | A_REVERSE);
  for(i=0;i<strlen(logo);i++, col++)
  {
    if(logo[i]=='%')
      mvwaddch(win, row, col, ' ');
    else if(logo[i]=='\n')
      { row++; col=0; }
  }
  wattroff(win, COLOR_PAIR(4) | A_REVERSE);

  wattron(win, A_BOLD);
  mvwaddstr(win, row+2, col-strlen(VERSION), VERSION);
  wattroff(win, A_BOLD);
  wrefresh(win);
}

void setup_fields(wins, addr, pass)
    struct login_windows *wins;
    char *addr;
    char *pass;
{
  wins->field[0] = new_field(1, 24, 2, 2, 4, 0);
  wins->field[1] = new_field(1, 24, 4, 2, 4, 0);
  wins->field[2] = NULL;

  /* Set field options */
  field_opts_off(wins->field[0], O_AUTOSKIP);
  field_opts_off(wins->field[0], O_BLANK);
  set_field_back(wins->field[0], A_UNDERLINE); 
  field_opts_off(wins->field[1], O_PUBLIC);
  field_opts_off(wins->field[1], O_AUTOSKIP);
  field_opts_off(wins->field[1], O_BLANK);
  set_field_back(wins->field[1], A_UNDERLINE); 
  set_field_buffer(wins->field[0], 0, addr);
  set_field_buffer(wins->field[1], 0, pass);
}    

void destroy_fields(wins)
    struct login_windows *wins;
{
  free_field(wins->field[0]);
  free_field(wins->field[1]);
}    

void draw_login(wins, max_y, max_x)
    struct login_windows *wins;
    int max_y;
    int max_x;
{
  wins->main = newwin(24, 43, (max_y-24)/2, (max_x-43)/2);
  wins->formwin = derwin(wins->main, 5, 26, 9, 13);
  statuswin = derwin(wins->main, 8, 42, 15, 1);
  refresh();

  print_logo(wins->main);

  /* Create the form and post it */
  wins->form = new_form(wins->field);
  set_form_win(wins->form, wins->formwin);
  set_form_sub(wins->form, derwin(wins->formwin, 5, 26, 0, 0));
  post_form(wins->form);
  wrefresh(wins->formwin);

  wattron(wins->main, A_BOLD);
  mvwaddstr(wins->main, 11, 5, _("Username:"));
  mvwaddstr(wins->main, 13, 5, _("Password:"));
  wattroff(wins->main, A_BOLD);
  wrefresh(wins->main);

  refresh();
}

void destroy_login(wins)
    struct login_windows *wins;
{
  delwin(wins->main);
  unpost_form(wins->form);
  free_form(wins->form);
  delwin(wins->formwin);
}

int handle_form(ch, form)
    int ch;
    FORM *form;
{
  switch(ch)
  {
    case KEY_DOWN:
    case '\t': /* Tab */
      form_driver(form, REQ_NEXT_FIELD);
      form_driver(form, REQ_END_LINE);
      break;
    case KEY_UP:
      form_driver(form, REQ_PREV_FIELD);
      form_driver(form, REQ_END_LINE);
      break;
    case KEY_LEFT:
      form_driver(form, REQ_PREV_CHAR);
      break;
    case KEY_RIGHT:
      form_driver(form, REQ_NEXT_CHAR);
      break;
    case KEY_BACKSPACE:
    case 127: /* Backspace in terminal.app */
      form_driver(form, REQ_PREV_CHAR); // Simulate backspace
      form_driver(form, REQ_DEL_CHAR);
      break;
    case '\n':
      if(field_index(current_field(form))==0)
      {
        form_driver(form, REQ_NEXT_FIELD);
        form_driver(form, REQ_END_LINE);
      }
      else
      {
        form_driver(form, REQ_LAST_FIELD); // Submit form
        return 1;
      }
      break;
    case KEY_F(10): /* Escape */
      endwin();
      fprintf(stderr, _("Quitting TMSNC.\n"));
      exit(0);
      break;
    case -1:
    case KEY_RESIZE:
      break;
    default:
      form_driver(form, ch);
      break;
  }
  return 0;
}


int UI_login(session, cf, dispatch, port)
    MSN_session *session;
    config *cf;
    char *dispatch;
    int port;
{
  int y, x;
  struct login_windows wins;
  char pass[80], *ptr;

  getmaxyx(stdscr, y, x);
  setup_fields(&wins, session->me.addr, cf->pass);
  draw_login(&wins, y, x);

  if(session->me.addr[0]!=0x0)
    set_current_field(wins.form, wins.field[1]);    
  else
    set_current_field(wins.form, wins.field[0]);

  while(session->sd<1)
  {
    if (cf->autologin==0)
    while(handle_form(getch(), wins.form)==0)
    {
      if(UI_is_killed())
      {
        destroy_login(&wins);
        destroy_fields(&wins);
        endwin();
        exit(0);
      }
      if(UI_is_resized())
      {
        destroy_login(&wins);
        endwin();
        initscr();
        erase();
        refresh();
        getmaxyx(stdscr, y, x);
        draw_login(&wins, y, x);
      }
      wrefresh(wins.formwin);
    }

    strncpy(session->me.addr,
            field_buffer(wins.field[0], 0),
            sizeof(session->me.addr)-1);

    strncpy(pass,
            field_buffer(wins.field[1], 0),
            sizeof(pass)-1);

    if((ptr = strchr(session->me.addr, ' '))!=NULL)
      *ptr = 0x0;
    if((ptr = strchr(pass, ' '))!=NULL)
      *ptr = 0x0;

    /* session->sd gets updated after running this function */
    MSN_init_session(dispatch,
                     port,
                     session,
                     pass,
                     &callback);

    /* zero the password field */
    set_field_buffer(wins.field[1], 0, "\0");
  }

  if(MSN_load_userlist(session, callback)<0)
    UI_err_exit(_("Couldn't retrieve the list of users"));

  destroy_login(&wins);
  destroy_fields(&wins);
  clear();
  return 0;
}
