/* TMSNC - Textbased MSN Client
 * Copyright (C) 2004 The IR Developer Group
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the IR Public Domain License as published by
 *  the IR Group; either version 1.6 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the IR Public Domain License
 * along with this program; if not, write to sanoix@gmail.com.
 */

#include "translate.h"
#include "common.h"

#ifdef ENABLE_NLS
#include <iconv.h>
#include <langinfo.h>

iconv_t from_cd;
iconv_t to_cd;
char *codeset;

void UI_iconv_init(cf)
    config *cf;
{
  if(cf->codeset[0]!='\0')
    codeset=cf->codeset;
  else
  {
    #ifdef CODESET
      codeset = nl_langinfo(CODESET);
    #else
      codeset = "ISO-8859-1";
    #endif
  }

  if ((from_cd = iconv_open(codeset, "UTF-8")) == (iconv_t) -1)
    UI_err_exit("iconv: character encoding translation not possible");
  if ((to_cd = iconv_open("UTF-8", codeset)) == (iconv_t) -1)
    UI_err_exit("iconv: character encoding translation not possible");
}

void UI_iconv_destroy()
{
  iconv_close(from_cd);
  iconv_close(to_cd);
}

int UI_convert_from_utf8(ibuf, isize, obuf, osize)
    char *ibuf;
    size_t isize;
    char *obuf;
    size_t osize;
{
  int rc=1;
  char *iptr = ibuf, *optr = obuf;
  size_t oleft=osize, ileft=isize;

  if(strcmp(codeset, "UTF-8")==0)
  {
    strncpy(obuf, ibuf, osize-1);
    return 0;
  }


  memset(obuf, 0x0, osize);

  while(ileft > 0 && oleft > 0)
  {
    rc = iconv(from_cd, &iptr, &ileft, &optr, &oleft);
    if(rc==-1)
    {
      *iptr='#';
      iptr++;
      ileft--;
    }
  }

  return rc;
}

int UI_convert_to_utf8(ibuf, isize, obuf, osize)
    char *ibuf;
    size_t isize;
    char *obuf;
    size_t osize;
{
  int rc=1;
  char *iptr = ibuf, *optr = obuf;
  size_t oleft=osize, ileft=isize;

  if(strcmp(codeset, "UTF-8")==0)
  {
    strncpy(obuf, ibuf, osize-1);
    return 0;
  }

  memset(obuf, 0x0, osize);

  while(ileft > 0 && oleft > 0)
  {
    rc = iconv(to_cd, &iptr, &ileft, &optr, &oleft);
    if(rc==-1)
    {
      *iptr='#';
      iptr++;
      ileft--;
    }
  }

  return rc;
}
#endif
